/*
		Chrono Cross CD extractor
		Copyright 2000-2001 by Yazoo (hamm@efrei.fr)
		Revision 0.1b ANSI C


	Features:

		Dump the complete content of Chrono Chross CD1/CD2 US and Japanese version
		It requires a iso named Chrono1.iso in the same directory

	Todo list:

		Find a way to locate end of the last file
		Better support for CD2
		Dump in subdirectory according to CD1/CD2 repartition
		Recompilation in Visual C++ 6 for disk speed optimisation
		Source comment and reorganisation
		Log feature (Optional since you can redirect output with > )
		Progression indicator
		Better detection of the ISO with error control
		Major code optimisation
		Integration in main Chrono Cross Hacking tool


*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <direct.h>


struct t_file_pointer
{
	char adress_byte_1;
	char adress_byte_2;
	char adress_byte_3;
	char file_type;
};

struct t_index_tab
{
	unsigned long address;
	unsigned char type;
};

long filesize(FILE *f_iso);
long check_iso(FILE *f_iso);
void read_file_pointer(FILE *f_iso,unsigned char user_data[]);
void sector_seek(FILE *f_iso, long sector);
void jump_over_cd_header(FILE *f_iso);
void jump_over_cd_edc_ecc(FILE *f_iso);
void file_dump(FILE *f_iso,unsigned long debut,unsigned long fin,long num,unsigned char user_data[],int stop_value);



int main(void)
{
	unsigned char user_data[2047];
	FILE *f_iso;

	printf("Chrono Cross File Exctactor\nBy Yazoo\n\n");

	f_iso = fopen("..\\iso\\cd1.iso", "rb");
	if(f_iso==NULL)
	{		
		printf("Unable to open the cd1.iso file....\n");
		return(1);
	}

	if (check_iso(f_iso)==1)
	{
		fclose(f_iso);
		printf("Warning:INVALID CHRONO CROSS ISO FILE!!!\n");
		printf("Make sure you are using a Genuine Chrono Cross CD1 iso.");
	}
	else
	{
		printf("Genuine Chrono Cross iso detected.\n");
	}
	printf("Entering file pointer read sequence\n");
	read_file_pointer(f_iso,user_data);
	fclose(f_iso);
	return 0;
}

long filesize(FILE *f_iso)
{
	long curpos, length;
	curpos = ftell(f_iso);
	fseek(f_iso, 0L, SEEK_END);
	length = ftell(f_iso);
	fseek(f_iso, curpos, SEEK_SET);
	return length;
}

long check_iso(FILE *f_iso)
{
	long length;
	long cd1_length=736651104;
	length=filesize(f_iso);
	printf("Filesize of cd1.iso is %ld bytes\n", length);
	if (length!=cd1_length)
	{
		return(1);
	}
	return(0);
}

void sector_seek(FILE *f_iso, long sector)
{
	long curpos;
	curpos=(2352*(long)sector);
	fseek(f_iso, curpos, SEEK_SET);
}

void jump_over_cd_header(FILE *f_iso)
{
	long curpos;
	curpos = ftell(f_iso);
	fseek(f_iso, (curpos+24), SEEK_SET);
}

void jump_over_cd_ecd_ecc(FILE *f_iso)
{
	long curpos;
	curpos = ftell(f_iso);
	fseek(f_iso, (curpos+280), SEEK_SET);
}

void read_user_data(FILE *f_iso,unsigned char user_data[])
{
	long i;
	for(i=0;i<2048;i++)
	{
		fread((char *)&user_data[i],1,1,f_iso);
	}
}

void read_file_pointer(FILE *f_iso,unsigned char user_data[])
{
	t_index_tab index_tab[10000];		//Standard max is 5617 but this is a drawback compatibility with the japanese version that have a few more files
	unsigned char c;
	long temp[4];
	long i;
	long j;
	long k;
	unsigned long indexer;
	long n_sector;
	unsigned long address;
	unsigned long address_temp;
	char type;
	FILE *f_out;
	FILE *f_index;
	f_out = fopen("file.tbl", "wb+");
	indexer=1;
	for(i=0;i<12;i++)
	{
		printf("%d\n",24+i);
		sector_seek(f_iso,24+i);
		jump_over_cd_header(f_iso);
		read_user_data(f_iso,user_data);
		for(j=0;j<2048;j=j+4)
		{
			temp[0]=user_data[j];
			temp[1]=user_data[j+1];
			temp[2]=user_data[j+2];
			temp[3]=user_data[j+3];
			if((temp[2]<79)||((temp[0]==0)&&(temp[1]==0)&&(temp[2]==0)&&(temp[3]==0)))
			{
				for(k=0;k<4;k++)
				{
					{
						fwrite(&temp[k],1,1,f_out);
					}
				}
			index_tab[indexer].address=((unsigned long)temp[0])+((long)256*(unsigned long)temp[1])+((long)256*(long)256*(long)temp[2]);
			index_tab[indexer++].type=temp[3];
			}
		}
	}
	index_tab[5619].address=((filesize(f_iso))/2352);
	printf("Index file generation complete.\n");
	fclose(f_out);
	for(i=1;i<=5618;i++)
	{
		address=index_tab[i].address;
		address_temp=index_tab[i+1].address;
		printf("\n%.0f - ",(float)address);
		printf("%.0f : ",(float)address_temp);
		file_dump(f_iso,address,address_temp,i-1,user_data,index_tab[i].type);
		address=address_temp;
	}
}

void file_dump(FILE *f_iso,unsigned long debut, unsigned long fin,long num,unsigned char user_data[],int stop_value)
{
	long i,j;
	long difference;
	char nom[100] ="";
	char *extention =".out";
	char nom_temp[100] ="";
	FILE *f_out;
	sprintf(nom_temp,"%d",num);
	
	strcat(nom,"..\\");

	if((num>=0)&&(num<=24))					//  0-   24 Misc
		strcat(nom,"MISC");

	if((num>=25)&&(num<=74))				//  25-  74 Character Faces
		strcat(nom,"CF");

	if((num>=75)&&(num<=125))				//  75- 125 Character Descriptions
		strcat(nom,"CD");

	if((num>=126)&&(num<=178))				// 126- 178 Rare objects TIM
		strcat(nom,"ROT");

	if((num>=179)&&(num<=185))				// 179- 185 Misc
		strcat(nom,"MISC");

	if((num>=186)&&(num<=189))				// 186- 189 Rare objects TIM
		strcat(nom,"ROT");

	if((num>=190)&&(num<=203))				// 190- 203 Misc
		strcat(nom,"MISC");

	if((num>=204)&&(num<=307))				// 204- 321 OverWorld Main Characters Models
		strcat(nom,"OMCM");

	if((num>=308)&&(num<=221))				// 190- 203 Misc
		strcat(nom,"MISC");

	if((num>=322)&&(num<=331))				// 322- 331 OverWorld Alternate Models
		strcat(nom,"OAM");

	if((num>=332)&&(num<=355))				// 332- 355 Misc
		strcat(nom,"MISC");

	if((num>=356)&&(num<=1966))				// 356-1966 Rooms
		strcat(nom,"Rooms");

	if((num>=1967)&&(num<=1977))			//1967-1977 Misc
		strcat(nom,"MISC");

	if((num>=1978)&&(num<=1992))			//1978-1992 Windows frame
		strcat(nom,"WF");

	if((num>=1993)&&(num<=2084))			//1993-2084 Character talking face
		strcat(nom,"CTF");

	if((num>=2085)&&(num<=2416))			//2085-2416 Misc
		strcat(nom,"MISC");

	if((num>=2417)&&(num<=2513))			//2415-2513 BattleFields
		strcat(nom,"BF");

	if((num>=2514)&&(num<=2542))			//2514-2542 Misc
		strcat(nom,"MISC");

	if((num>=2543)&&(num<=3054))			//2543-3054 Battlescript
		strcat(nom,"BS");

	if((num>=3055)&&(num<=3707))			//3054-3707 Misc
		strcat(nom,"MISC");

	if((num>=3708)&&(num<=3759))			//3708-3759 Small Character Layer
		strcat(nom,"SCL");

	if((num>=3760)&&(num<=5597))			//3760-5597 Misc
		strcat(nom,"MISC");

	if(num>=5598)							//5598-5918 Video
		strcat(nom,"V");

	strcat(nom,"\\");

	mkdir(nom);

	strcat(nom,"disk1\\");

	mkdir(nom);

	if(num<10)
		strcat(nom,"0");
	if(num<100)
		strcat(nom,"0");
	if(num<1000)
		strcat(nom,"0");
	strcat(nom,nom_temp);

	strcat(nom,extention);
	f_out = fopen(nom, "wb+");
	difference=fin-debut;
	sector_seek(f_iso,debut);
	for(i=0;i<difference;i++)
	{
		sector_seek(f_iso,debut+i);
		jump_over_cd_header(f_iso);
		read_user_data(f_iso,user_data);
		if(i!=(difference-1))
		{
			for(j=0;j<2048;j++)
			{
				fwrite(&user_data[j],1,1,f_out);
			}
		}
		else
		{
			for(j=0;j<(2048-8*stop_value);j++)
			{
				fwrite(&user_data[j],1,1,f_out);
			}
		}
	}
	fclose(f_out);
	printf("dumped file number %d ",num);

	
	if((num>=0)&&(num<=24))
		printf("Misc");	
	if((num>=25)&&(num<=74))
		printf("Character face");
	if((num>=75)&&(num<=125))
		printf("Character description");
	if((num>=126)&&(num<=178))
		printf("Rare object TIM");
	if((num>=179)&&(num<=185))
		printf("Misc");
	if((num>=186)&&(num<=189))
		printf("Rare object TIM");
	if((num>=190)&&(num<=203))
		printf("Misc");
	if((num>=204)&&(num<=307))
		printf("OverWorld Main Characters Models");
	if((num>=308)&&(num<=321))
		printf("Misc");
	if((num>=322)&&(num<=331))
		printf("OverWorld Alternate Models");
	if((num>=1978)&&(num<=1992))
		printf("Windows frame");
	if((num>=332)&&(num<=355))
		printf("Misc");
	if((num>=356)&&(num<=1966))
		printf("Rooms");
	if((num>=1967)&&(num<=1977))
		printf("Misc");
	if((num>=1978)&&(num<=1992))
		printf("Windows frame");
	if((num>=1993)&&(num<=2084))
		printf("Character talking face");
	if((num>=2085)&&(num<=2416))
		printf("Misc");
	if((num>=2417)&&(num<=2513))
		printf("Battlefield");
	if((num>=2514)&&(num<=2542))
		printf("Misc");
	if((num>=2543)&&(num<=3054))
		printf("Battlefield script");
	if((num>=3055)&&(num<=3707))
		printf("Misc");
	if((num>=3708)&&(num<=3759))
		printf("Small Character Layer");
	if((num>=3760)&&(num<=5597))
		printf("Misc");
	if(num>=5598)
		printf("Video");


}
